function dataout = bmelab(wx, wy, dt, sys, task)
%
% AUTHOR
%   Jasper Schuurmans,
%   Laboratory for Neuromuscular Control,
%   Delft University of Technology.
%
% ACKNOWLEDGEMENT
%   This program is based on MMSLab by Max Mulder.
%
% REVISION HISTORY (date, author, changes):
%   28 apr 2008, JS, version 1.1, added two-dimensional tasks.
%   19 jul 2007, JS, version 1.0, added opengl renderer
%   15 jul 2007, JS, beta version.
%
% PURPOSE
%   This program runs an experiment on a human controller performing a
%   pursuit or compensatory task with the mouse.
%
% PURSUIT TASK
%   The human controls a virtual system ('sys') and is ordered to follow a
%   circular target. The movements of the target are prescribed by inputs
%   'wx' and 'wy', which are vectors with position samples (sample time
%   'dt'). The pursuit task as a block scheme:
%
%       +          --------          --------
% --(w)->O--(e)-->| human  |--(u)-->|  sys   |--(x)------>
%        ^ -       --------          --------         |
%        |                                            |
%        ----------------------------------------------
%
% COMPENSATORY TASK 
%   The human controls a virtual system ('sys') and is ordered to maintain
%   a fixed reference position while being perturbed by a visual
%   disturbance 'w'. This means that there is a constant setpoint of zero
%   and that the disturbance acts on the cursor position.
%   In a block scheme:
%
%                                                      |(w)
%                                                      |   
%       +          --------          --------          v +
%  0 --->O--(e)-->| human  |--(u)-->|  sys   |--(x)----O----->
%        ^ -       --------          --------         +   |
%        |                                                |
%        --------------------------------------------------
%
% USAGE:
%   dataout = bmeLab(wx, wy, dt, sys, task)
%
%   wx:   Disturbance signal in the horizontal direction (range: [-1..1]).
%   wy:   Disturbance signal in the horizontal direction (range: [-1..1]).        
%   dt:   Sample time used in 'wx' and 'wy' (in seconds).
%   sys:  Transfer function of controlled system. Create with command tf().
%   task: Task selection. Can be 'pursuit' or 'compensate'.
%
%   dataout: structure containing measured data with following fields:
%       .t :   time
%       .ux:   user input in horizontal direction (mouse)
%       .uy:   user input in vertical direction (mouse)
%       .xx:   system output in horizontal direction
%       .xy:   system output in vertical direction
%       .wx:   disturbance in horizontal direction
%       .wy:   disturbance in vertical direction 
%       .ex:   error in horizontal direction
%       .ey:   error in vertical direction
%
%   NOTE: 'wx' and 'wy' need to have the same length and the same sample
%         time. To select a uni-directional task, choose either 'wx' or
%         'wy' as an all-zeros vector.
%

%% Check inputs

% Show help if no inputs
if nargin == 0
    help(mfilename)
    return
elseif nargin ~= 5
    error('Wrong number of input arguments. BMELAB needs 5 inputs arguments.')
end

% Disturbance wx
if ~isvector(wx)
    error('Input ''wx'' must be a vector.')
else
    if max(abs(wx)) > 1
        error('Values of input ''wx'' must lie in the interval [-1..1]')
    end
end

% Disturbance wy
if ~isvector(wy)
    error('Input ''wy'' must be a vector.')
else
    if max(abs(wy)) > 1
        error('Values of input ''wy'' must lie in the interval [-1..1]')
    end
end

% wx and wy must have same length
if length(wy)~=length(wx)
    error('Disturbance signals ''wx'' and ''wy'' must have the same length')
end

% Check if a 1-D task is selected
if (nnz(wx) == 0) && (nnz(wy) == 0)
    % Assume two-dimensional movement
    yDirOnly = 0;
    xDirOnly = 0;
elseif nnz(wx) == 0
    % Assume one-dimensional movement in y-dir
    yDirOnly = 1;
    xDirOnly = 0;
elseif nnz(wy) == 0
    % Assume one-dimensional movement in x-dir
    yDirOnly = 0;
    xDirOnly = 1;
else
    yDirOnly = 0;
    xDirOnly = 0;
end

% Sample time
if ~isscalar(dt)
    error('Input ''dt'' (sample time) must be a scalar.')
end

% Controlled system
if ~strcmp(class(sys),'tf')
    error('Input ''sys'' (system''s transfer function) must be of class ''tf''.')
end

% Task
if ~(strcmp(task, 'pursuit') || strcmp(task, 'compensate'))
    error('Unknown value for input ''task''. Options are ''compensate'' or ''pursuit''')
end

%% Convert transfer function 'sys' to discrete state space
% Original 'sys' is SISO, it must be converted to MIMO (x and y
% directions)
sysDisc = c2d(ss(sys), dt);

sysA = blkdiag(sysDisc.A, sysDisc.A);
sysB = blkdiag(sysDisc.B, sysDisc.B);
sysC = blkdiag(sysDisc.C, sysDisc.C);
sysD = blkdiag(sysDisc.D, sysDisc.D);
sysState = zeros(size(sysA, 2), 1);

%% Init figure
% Make figure
hfig = figure;
set(hfig, 'units', 'normalized',...
    'position', [0 0 1 1],...
    'menubar', 'none',...
    'renderer','painters',...
    'name', 'bmeLab',...
    'numbertitle', 'off')

% Add target marker
htarget = plot(0, 0, 'go','markersize',30, 'linewidth', 2, 'buttondownfcn', 'uiresume'); hold on

% Add position (of controlled system) marker
hpos = plot(0, 0, 'r+', 'markersize', 20, 'linewidth', 2, 'buttondownfcn', 'uiresume');

% Add horizontal mouse input indicator on bottom of screen
if ~yDirOnly
    plot([-0.505, 0.505],  [-0.8,-0.8],  'w', 'linewidth', 8);
    plot([-0.5, 0.5],  [-0.8,-0.8],  'k', 'linewidth', 6);
    hmousex = plot([0,0],  [-0.8,-0.8],  'w', 'linewidth', 4);
end

% Add vertical mouse input indicator on right of screen
if ~xDirOnly
    plot([0.8, 0.8], [-0.505, 0.505], 'w', 'linewidth', 8);
    plot([0.8, 0.8], [-0.5, 0.5], 'k', 'linewidth', 6);
    hmousey = plot([0.8, 0.8], [0,0], 'w', 'linewidth', 4);
end

% Axis markup
axis([-1 1 -1 1])
set(gca, 'color', 'k', ...
    'units', 'normalized', ...
    'position', [0 0 1 1],...
    'xtick', [],...
    'ytick', [],...
    'drawmode', 'fast')

% Init data storage
N = length(wx);
data = zeros(N, 7);
drawnow

%% Countdown for start
delay = 3; % Countdown delay [s]

% Display task message and start instructions
switch task
    case 'pursuit'
        htext = text(0, 0.5, {'Task: pursuit.  Keep the crossed marker on the circular target';''; 'Click the target to start...'}, 'fontsize', 25, 'color', 'w', 'horizontalalignment', 'center');
    case 'compensate'
        htext = text(0, 0.5, {'Task: compensate. Keep the crossed marker on the circular target';''; 'Click the target to start...'}, 'fontsize', 25, 'color', 'w', 'horizontalalignment', 'center');
end

% Wait for mouse click on target marker or system marker
uiwait

% Hide mouse pointer
set(gcf, 'pointer', 'custom',...
   'pointershapecdata', nan(16,16))

% Countdown
set(htext, 'string', num2str(delay));
tic
for kk = 1:delay;
    drawnow
    while (toc < kk)
    end
    set(htext,'string', num2str(delay-kk))
end
set(htext, 'string', '')
drawnow

%% Start sim
% Set mouse gain
mouseGain = 1;

% Start timer
tic
for ii = 1:N
    % Get mouse input
    umouse = mouseGain .* GetMouseInput .* ~[yDirOnly; xDirOnly];
    umouse = sign(umouse) .* min(1, abs(umouse));

    % Update target position
    switch task
        case 'pursuit'
            set(htarget, 'xdata', wx(ii), 'ydata', wy(ii));
        case 'compensate'
            % Do nothing
    end

    % Update controlled system states
    sysState = sysA * sysState + sysB * umouse;
    sysOut = sysC * sysState + sysD * umouse;

    % Store data
    data(ii,:) = [toc, umouse(1), umouse(2), sysOut(1), sysOut(2), wx(ii), wy(ii)];

    % Update screen
    switch task
        case 'pursuit'
            % Plot system output
            set(hpos, 'xdata', sysOut(1), 'ydata', sysOut(2));
        case 'compensate'
            % Plot system output
            set(hpos, 'xdata', sysOut(1) + wx(ii), 'ydata', sysOut(2) + wy(ii));
    end
    if ~yDirOnly
        set(hmousex, 'xdata', [0, 0.5 * umouse(1)]);
    end
    if ~xDirOnly
        set(hmousey, 'ydata', [0, 0.5 * umouse(2)]);
    end
    drawnow

    % Wait until next sample hit
    while (toc < (ii) * dt)
    end
end

%% End sim
set(htext, 'string', 'Done...')
pause(1);
close(hfig)

%% Analyze, check and return data
t = data(:,1);
ux = data(:,2) .* ~yDirOnly;
uy = data(:,3) .* ~xDirOnly;
xx = data(:,4) .* ~yDirOnly;
xy = data(:,5) .* ~xDirOnly;
wx = data(:,6) .* ~yDirOnly;
wy = data(:,7) .* ~xDirOnly;

% Sample times
trueDt = diff(t);
meanDt = mean(trueDt);

% Check if computer has managed to obtain requested sample time
if (meanDt/dt) > 1.05 % 5 percent error allowed
    warning('bmeLab:sampling','The requested sampling time ''dt'' might have been too small for your processor speed.\n Experiment outcome is unreliable!')
    fprintf('Requested sample time: %f (%0.4f Hz) \n', dt, 1/dt)
    fprintf('Mean true sample time: %f (%0.4f Hz) \n', meanDt, 1/meanDt)
end

% Interpolate data to the requested sample times
% This removes the variance in sampling times
tInt = (0:N-1)' * dt;
uxInt = interp1(t, ux, tInt); uxInt(1) = 0;
uyInt = interp1(t, uy, tInt); uyInt(1) = 0;
xxInt = interp1(t, xx, tInt); xxInt(1) = 0;
xyInt = interp1(t, xy, tInt); xyInt(1) = 0;
wxInt = interp1(t, wx, tInt); wxInt(1) = 0;
wyInt = interp1(t, wy, tInt); wyInt(1) = 0;

% Return data
dataout.t = tInt;
dataout.ux = uxInt;
dataout.uy = uyInt;
dataout.xx = xxInt;
dataout.xy = xyInt;
dataout.wx = wxInt;
dataout.wy = wyInt;
dataout.ex = wxInt - xxInt;
dataout.ey = wyInt - xyInt;
end

function mouseLoc = GetMouseInput()
% Determine mouse location and convert to the range
% (minHor...maxhor) X (minVer...maxVer)

% Range settings
minHor = -1;
maxHor = 1;
minVer = -1;
maxVer = 1;

% Current cursor location
screenSize = get(0, 'monitorpositions');
curPos = get(0, 'pointerlocation');

% Convert to proper range
hor = (curPos(1) - screenSize(1)) / (screenSize(3) - screenSize(1)) * (maxHor - minHor) + minHor;
ver = (curPos(2) - screenSize(2)) / (screenSize(4) - screenSize(2)) * (maxVer - minVer) + minVer;

mouseLoc = [hor; ver];
end